@@ -1,48 +1,16 @@
-from time import time
+import inspect
 from collections import defaultdict
-from inspect import stack
+from time import time
 
+import wrapt
 from django.dispatch import Signal
-from django.template import Template
 from django.utils.translation import ugettext_lazy as _
 
 from debug_toolbar.panels import Panel
+from debug_toolbar.panels.sql.utils import contrasting_color_generator
 
 
-def dummy_color_generator():
-    while True:
-        yield '#bbbbbb'
-
-# Contrasting_color_generator is available since debug toolbar version 1.1.
-try:
-    from debug_toolbar.panels.sql.utils import contrasting_color_generator
-except ImportError:
-    # Support older versions of debug toolbar
-    contrasting_color_generator = dummy_color_generator
-
-template_rendered = Signal(
-    providing_args=['instance', 'start', 'end', 'level'])
-
-
-def template_render_wrapper(self, context):
-    result = None
-    if hasattr(Template, 'tp_saved_render'):
-        t_start = time()
-        try:
-            result = Template.tp_saved_render(self, context)
-        finally:
-            t_end = time()
-
-            template_rendered.send(
-                sender=Template, instance=self, start=t_start, end=t_end,
-                level=len(stack()))
-
-    return result
-
-# Wrap the original Template.render
-if not hasattr(Template, 'tp_saved_render'):
-    Template.tp_saved_render = Template.render
-    Template.render = template_render_wrapper
+template_rendered = Signal(providing_args=['instance', 'start', 'end', 'level'])
 
 
 class TemplateProfilerPanel(Panel):
@@ -51,18 +19,63 @@ class TemplateProfilerPanel(Panel):
     '''
 
     template = 'template_profiler_panel/template.html'
-    colors = None
-    templates = None
-    color_generator = None
-    t_min = 0
-    t_max = 0
-    total = 0
 
     def __init__(self, *args, **kwargs):
         self.colors = {}
         self.templates = []
         self.color_generator = contrasting_color_generator()
-        return super(TemplateProfilerPanel, self).__init__(*args, **kwargs)
+        self.t_min = 0
+        self.t_max = 0
+        self.total = 0
+        self.monkey_patch_template_classes()
+        self.is_enabled = False
+        template_rendered.connect(self.record)
+        super(TemplateProfilerPanel, self).__init__(*args, **kwargs)
+
+    have_monkey_patched_template_classes = False
+
+    @classmethod
+    def monkey_patch_template_classes(cls):
+        if cls.have_monkey_patched_template_classes:
+            return
+
+        from django.template import Template as DjangoTemplate
+        template_classes = [DjangoTemplate]
+
+        try:
+            from jinja2 import Template as Jinja2Template
+        except ImportError:
+            pass
+        else:
+            template_classes.append(Jinja2Template)
+
+        @wrapt.decorator
+        def render_wrapper(wrapped, instance, args, kwargs):
+            start = time()
+            result = wrapped(*args, **kwargs)
+            end = time()
+
+            stack_depth = 1
+            current_frame = inspect.currentframe()
+            while True:
+                current_frame = current_frame.f_back
+                if current_frame is None:
+                    break
+                stack_depth += 1
+
+            template_rendered.send(
+                sender=instance.__class__,
+                instance=instance,
+                start=start,
+                end=end,
+                level=stack_depth,
+            )
+            return result
+
+        for template_class in template_classes:
+            template_class.render = render_wrapper(template_class.render)
+
+        cls.have_monkey_patched_template_classes = True
 
     @property
     def nav_title(self):
@@ -80,7 +93,21 @@ class TemplateProfilerPanel(Panel):
     def _get_color(self, level):
         return self.colors.setdefault(level, next(self.color_generator))
 
-    def record(self, sender, instance, start, end, level, **kwargs):
+    def record(self, instance, start, end, level, **kwargs):
+        if not self.enabled:
+            return
+
+        template_name = instance.name
+        # Logic copied from django-debug-toolbar:
+        # https://github.com/jazzband/django-debug-toolbar/blob/5d095f66fde8f10b45a93c0b35be0a85762b0458/debug_toolbar/panels/templates/panel.py#L77
+        is_skipped_template = isinstance(template_name, str) and (
+            template_name.startswith("debug_toolbar/")
+            or template_name.startswith(
+                tuple(self.toolbar.config["SKIP_TEMPLATE_PREFIXES"])
+            )
+        )
+        if is_skipped_template:
+            return
 
         bg = self._get_color(level)
         text = '#ffffff' if int(bg[1:], 16) < 0x8fffff else '#000000'
@@ -91,18 +118,19 @@ class TemplateProfilerPanel(Panel):
             'end': end,
             'time': (end - start) * 1000.0,
             'level': level,
-            'name': instance.name,
+            'name': template_name,
             'color': color,
         })
 
     def enable_instrumentation(self):
-        template_rendered.connect(self.record)
+        self.is_enabled = True
 
     def disable_instrumentation(self):
-        template_rendered.disconnect(self.record)
+        self.is_enabled = False
 
     def _calc_p(self, part, whole):
-        return (part / whole) * 100.0
+        # return the percentage of part or 100% if whole is zero
+        return (part / whole) * 100.0 if whole else 100.0
 
     def _calc_timeline(self, start, end):
         result = {}
@@ -117,7 +145,9 @@ class TemplateProfilerPanel(Panel):
 
         return result
 
-    def process_response(self, request, response):
+    def process_request(self, request):
+        response = super(TemplateProfilerPanel, self).process_request(request)
+
         summary = defaultdict(float)
 
         # Collect stats
@@ -142,3 +172,5 @@ class TemplateProfilerPanel(Panel):
         self.record_stats(
             {'templates': sorted(self.templates, key=lambda d: d['start']),
              'summary': sorted(summary.items(), key=lambda t: -t[1])})
+
+        return response
